#!/usr/bin/python

from ion.plugin import *
import os
import partek.plugin
import partek.util


class PartekFlowUploader(IonPlugin):
    """Partek Flow plugin for TorrentSuite.

    The Partek Flow plugin handles importing files into Flow,
    while also providing QA/QC and ERCC analysis.
    """

    # Plugin Configuration
    version = "1.02"
    runtypes = [RunType.FULLCHIP, RunType.COMPOSITE]
    runlevels = [RunLevel.LAST]
    features = [Feature.EXPORT]

    pluginName = "PartekFlowUploader"

    START_PLUGIN_JSON = 'startplugin.json'

    def launch(self, data=None):
        plugin = partek.plugin.Plugin()
        plugin.configuration = self.load_configuration()
        return plugin.launch(data)

    def load_configuration(self):
        result = partek.plugin.Configuration()
        config = partek.util.load_json(self.START_PLUGIN_JSON)
        runinfo = config['runinfo']
        plugin_config = config['pluginconfig']

        result.basecaller_dir = runinfo['basecaller_dir']
        result.alignment_dir = runinfo['alignment_dir']
        result.barcode_filter = os.path.join(runinfo['basecaller_dir'], 'barcodeFilter.txt')
        result.run_name = config['expmeta']['run_name']
        result.reference_path = os.getenv('TSP_FILEPATH_GENOME_FASTA')
        result.result_dir = runinfo['results_dir']
        result.plugin_dir = runinfo['plugin_dir']
        result.flow_server = plugin_config['server']
        result.flow_username = plugin_config['username']
        result.flow_password = plugin_config['password']
        if 'project' in plugin_config:
            result.flow_project = plugin_config['project']
        elif 'userInput' in runinfo['plugin']:
            result.flow_project = runinfo['plugin']['userInput']['project']
        else:
            result.flow_project = result.flow_username + '-' + result.run_name

        return result


if __name__ == "__main__":
    PluginCLI(PartekFlowUploader())
