import subprocess
import logging


class ConnectionError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return str(self.value)


class CurlCommand(object):
    """Simple class to build and run a curl command.

    """

    _params = {}
    _destination = ""
    log = logging.getLogger('CurlCommand')

    def __init__(self, server="", params=None):
        if params:
            self._params = params
        self._destination = server

    @property
    def destination(self):
        return self._destination

    @property
    def params(self):
        return self._params

    def execute(self, timeout=None):
        """ Executes the built CurlCommand. If timeout (seconds) is specified, a ConnectionError
        will be raised if the connection cannot be established before timeout.

        """
        arguments = ['curl', '-k', '-s', '-S']
        if timeout:
            arguments.extend(['--connect-timeout', timeout])
        for key, value in self._params.iteritems():
            arguments.extend(["-F", key + "=" + value])
        if self._destination:
            arguments.append(self._destination)
        p = subprocess.Popen(arguments, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        std_out, std_err = p.communicate()

        if std_err:
            self._handle_std_error(std_err)
        return std_out

    def _handle_std_error(self, std_err):
        self.log.error('Error while running curl: %s', std_err)
        # Try to provide an informative message
        if std_err.startswith('curl: (28)'):
            raise ConnectionError('Connection timeout while trying to reach host')
        elif std_err.startswith('curl: (6)'):
            raise ConnectionError("Could not resolve destination - " + self._destination)
        else:
            raise ConnectionError(std_err)
