import logging
import os

from partek import util

log = logging.getLogger('barcode_filter')


def load_filter_from_json(json_file):
    _barcodes = []
    barcodes = _barcodes_read(util.load_json(json_file))
    if barcodes:
        _barcodes += [bc for bc in barcodes if not 'filtered' in bc]
        _barcodes += [bc for bc in barcodes if 'filtered' in bc and not bc['filtered']]

    result = ['nomatch']
    if _barcodes:
        result += [bc['barcode_name'] for bc in _barcodes if 'barcode_name' in bc]
    return result


def load_filter_from_file(filter_file):
    result = ['nomatch']
    if os.path.exists(filter_file):
        with open(filter_file) as f:
            f.readline()  # Discard the first line
            file_content = f.read()
            for line in file_content.split('\n'):
                if line:
                    barcode_id, barcode, num_reads, include = tuple(line.split(','))
                    if int(include):
                        result.append(barcode_id)
    return result


def filter_files(bam_list, barcode_list):
    result = []
    log.info('Filtering bam files')
    for barcode in barcode_list:
        log.info('Current filter: %s', barcode)
        for bam in bam_list:
            if os.path.basename(bam).startswith(barcode):
                log.info('Retaining: %s', bam)
                result.append(bam)
    if not result:
        return bam_list
    return result


def _barcodes_read(barcodes):
    """Return a list of relevant barcodes

    """
    if not barcodes:
        return False

    try:
        for dataset in barcodes.get('datasets', []):
            file_prefix = dataset['file_prefix']
            for rg in dataset.get('read_groups', []):
                if rg in barcodes.get('read_groups', {}):
                    barcodes['read_groups'][rg]['file_prefix'] = file_prefix
    except KeyError:
        return False

    barcode_list = []
    read_groups = barcodes.get('read_groups', {})
    barcode_list += [value for key, value in read_groups.iteritems()]
    return sorted(barcode_list, key=lambda x: x.get('index', 0))
