import unittest
import logging
from partek import CurlCommand

logging.basicConfig()


class TestCurlCommand(unittest.TestCase):
    def test_empty_creation(self):
        curl_command = CurlCommand.CurlCommand()
        self.assertFalse(curl_command.destination, 'Destination should be empty')
        self.assertFalse(curl_command.params, 'Param dictionary should be empty')

    def test_destination_only_creation(self):
        curl_command = CurlCommand.CurlCommand('http://localhost:8080')
        self.assertEqual(curl_command.destination, 'http://localhost:8080',
                         'Destination should be: http://localhost:8080')
        self.assertFalse(curl_command.params, 'Param dictionary should be empty')

    def test_destination_and_params(self):
        curl_command = CurlCommand.CurlCommand('http://localhost:8080', {'username': 'test', 'password': 'test'})
        self.assertEqual(curl_command.destination, 'http://localhost:8080',
                         'Destination should be: http://localhost:8080')
        self.assertEqual(curl_command.params, {'username': 'test', 'password': 'test'}, 'Params set correctly')

    def test_execution_fail(self):
        curl_command = CurlCommand.CurlCommand('http://thisisdefinitelyafakeaddress.partek.com')
        self.assertRaisesRegexp(CurlCommand.ConnectionError, 'Could.n.t resolve.*', curl_command.execute)

        curl_command = CurlCommand.CurlCommand('http://localhost:9990')
        self.assertRaisesRegexp(CurlCommand.ConnectionError, 'Failed.* connect', curl_command.execute)

        curl_command = CurlCommand.CurlCommand('http://192.168.1.1')
        self.assertRaisesRegexp(CurlCommand.ConnectionError, 'Connection timeout while trying to reach host',
                                curl_command.execute, '1')

    def test_execution_success(self):
        curl_command = CurlCommand.CurlCommand('www.google.com')
        curl_command.execute()
