import unittest
import ConfigParser
import os
import logging

from partek import PartekSender
from partek import CurlCommand

logging.basicConfig()


class TestPartekSender(unittest.TestCase):
    _host = ''
    _user = ''
    _pass = ''
    _hash = ''

    def setUp(self):
        config = ConfigParser.ConfigParser()
        config.read(os.path.join(os.path.dirname(__file__), 'test_data', 'test.cfg'))
        self._host = config.get('flow_connection', 'host')
        self._user = config.get('flow_connection', 'user')
        self._pass = config.get('flow_connection', 'password')
        self._hash = config.get('flow_connection', 'password_hash')

    def test_bad_creation(self):
        self.assertRaisesRegexp(CurlCommand.ConnectionError, 'Could.*n.*t resolve.*', PartekSender.PartekSender)
        # self.assertTrue('Could not resolve host' in cm.exception.value)

    def test_bad_validation(self):
        with self.assertRaises(PartekSender.ValidationError) as cm:
            PartekSender.PartekSender(self._host, 'bad_user', 'bad_password')
        self.assertTrue('Invalid username or password' in cm.exception.value)

    def test_good_creation(self):
        sender = PartekSender.PartekSender(self._host, self._user, self._pass)
        self.assertEqual(sender.server, self._host)
        self.assertEqual(sender.user, self._user)
        self.assertEqual(sender._PartekSender__password, self._hash)
        self.assertEqual(sender.encrypt, '1')
        self.assertEqual(sender.SALT, 'That rug *really* tied the room together, dude. :) ')

    def test_cleaning_server_name(self):
        sender = PartekSender.PartekSender(self._host + '/flow', self._user, self._pass)
        self.assertEqual(sender.server, self._host)

if __name__ == '__main__':
    unittest.main()
