import json
import os
import unittest
from partek.barcode_filter import filter as bc_filter


class TestBarcodeFilter(unittest.TestCase):
    def test_barcodes_read(self):
        json_data = json.load(open(os.path.join(os.path.dirname(__file__),
                                                'test_data',
                                                'test_datasets_basecaller.json')))

        observed_barcodes = bc_filter._barcodes_read(json_data)
        expected_barcodes = [
            {'index': 0, 'file_prefix': 'nomatch_rawlib'},
            {'barcode_name': 'IonXpress_001', 'index': 1, 'file_prefix': 'IonXpress_001_rawlib', 'filtered': False},
            {'barcode_name': 'IonXpress_002', 'index': 2, 'file_prefix': 'IonXpress_002_rawlib', 'filtered': True},
            {'barcode_name': 'IonXpress_003', 'index': 3, 'file_prefix': 'IonXpress_003_rawlib', 'filtered': False},
            {'barcode_name': 'IonXpress_004', 'index': 4, 'file_prefix': 'IonXpress_004_rawlib', 'filtered': True},
            {'barcode_name': 'IonXpress_005', 'index': 5, 'file_prefix': 'IonXpress_005_rawlib', 'filtered': False},
            {'barcode_name': 'IonXpress_006', 'index': 6, 'file_prefix': 'IonXpress_006_rawlib', 'filtered': True},
            {'barcode_name': 'IonXpress_007', 'index': 7, 'file_prefix': 'IonXpress_007_rawlib', 'filtered': False},
            {'barcode_name': 'IonXpress_008', 'index': 8, 'file_prefix': 'IonXpress_008_rawlib', 'filtered': True},
            {'barcode_name': 'IonXpress_009', 'index': 9, 'file_prefix': 'IonXpress_009_rawlib', 'filtered': False},
            {'barcode_name': 'IonXpress_010', 'index': 10, 'file_prefix': 'IonXpress_010_rawlib', 'filtered': True},
            {'barcode_name': 'IonXpress_011', 'index': 11, 'file_prefix': 'IonXpress_011_rawlib', 'filtered': False},
            {'barcode_name': 'IonXpress_012', 'index': 12, 'file_prefix': 'IonXpress_012_rawlib', 'filtered': True},
            {'barcode_name': 'IonXpress_013', 'index': 13, 'file_prefix': 'IonXpress_013_rawlib', 'filtered': False},
            {'barcode_name': 'IonXpress_014', 'index': 14, 'file_prefix': 'IonXpress_014_rawlib', 'filtered': True},
            {'barcode_name': 'IonXpress_015', 'index': 15, 'file_prefix': 'IonXpress_015_rawlib', 'filtered': False},
            {'barcode_name': 'IonXpress_016', 'index': 16, 'file_prefix': 'IonXpress_016_rawlib', 'filtered': True}
        ]
        self.assertListEqual(expected_barcodes, observed_barcodes)

    def test_load_filter_from_file(self):
        expected_barcodes = [
            'nomatch', 'IonXpress_001', 'IonXpress_003', 'IonXpress_005',
            'IonXpress_007', 'IonXpress_009', 'IonXpress_011', 'IonXpress_013', 'IonXpress_015'
        ]

        observed_barcodes = bc_filter.load_filter_from_file(os.path.join(os.path.dirname(__file__),
                                                                         'test_data',
                                                                         'test_barcodeFilter.txt'))
        self.assertListEqual(expected_barcodes, observed_barcodes)

    def test_load_filter_from_json(self):
        expected_barcodes = [
            'nomatch',
            'IonXpress_001', 'IonXpress_003', 'IonXpress_005', 'IonXpress_007',
            'IonXpress_009', 'IonXpress_011', 'IonXpress_013', 'IonXpress_015'
        ]

        observed_barcodes = bc_filter.load_filter_from_json(os.path.join(os.path.dirname(__file__),
                                                                         'test_data',
                                                                         'test_datasets_basecaller.json'))
        self.assertListEqual(expected_barcodes, observed_barcodes)

    def test_filter_files(self):
        bam_list = [
            '/fake/path/to/IonXpress_001_rawlib.bam',
            '/fake/path/to/IonXpress_002_rawlib.bam',
            '/fake/path/to/IonXpress_003_rawlib.bam',
            '/fake/path/to/IonXpress_004_rawlib.bam',
            '/fake/path/to/IonXpress_005_rawlib.bam',
            '/fake/path/to/IonXpress_006_rawlib.bam',
            '/fake/path/to/IonXpress_007_rawlib.bam',
            '/fake/path/to/IonXpress_008_rawlib.bam',
            '/fake/path/to/IonXpress_009_rawlib.bam',
            '/fake/path/to/IonXpress_010_rawlib.bam',
            '/fake/path/to/IonXpress_011_rawlib.bam',
            '/fake/path/to/IonXpress_012_rawlib.bam',
            '/fake/path/to/IonXpress_013_rawlib.bam',
            '/fake/path/to/IonXpress_014_rawlib.bam',
            '/fake/path/to/IonXpress_015_rawlib.bam',
            '/fake/path/to/IonXpress_016_rawlib.bam',
            '/fake/path/to/nomatch_rawlib.bam'
        ]

        observed_filtered_list = bc_filter.filter_files(bam_list, [])
        self.assertListEqual(bam_list, observed_filtered_list)

        filter_list = [
            'IonXpress_001', 'IonXpress_003', 'IonXpress_005', 'IonXpress_007',
            'IonXpress_009', 'IonXpress_011', 'IonXpress_013', 'IonXpress_015'
        ]
        expected_filtered_list = ['/fake/path/to/' + bc + '_rawlib.bam' for bc in filter_list]
        observed_filtered_list = bc_filter.filter_files(bam_list, filter_list)
        self.assertListEqual(expected_filtered_list, observed_filtered_list)

    def test_filter_files_from_json(self):
        expected_result = [
            '/fake/path/to/nomatch_rawlib.bam',
            '/fake/path/to/IonXpress_001_rawlib.bam',
            '/fake/path/to/IonXpress_003_rawlib.bam',
            '/fake/path/to/IonXpress_005_rawlib.bam',
            '/fake/path/to/IonXpress_007_rawlib.bam',
            '/fake/path/to/IonXpress_009_rawlib.bam',
            '/fake/path/to/IonXpress_011_rawlib.bam',
            '/fake/path/to/IonXpress_013_rawlib.bam',
            '/fake/path/to/IonXpress_015_rawlib.bam'
        ]

        barcode_filter = bc_filter.load_filter_from_json(
            os.path.join(os.path.dirname(__file__), 'test_data', 'test_datasets_basecaller.json')
        )
        bam_list = [
            '/fake/path/to/IonXpress_001_rawlib.bam',
            '/fake/path/to/IonXpress_002_rawlib.bam',
            '/fake/path/to/IonXpress_003_rawlib.bam',
            '/fake/path/to/IonXpress_004_rawlib.bam',
            '/fake/path/to/IonXpress_005_rawlib.bam',
            '/fake/path/to/IonXpress_006_rawlib.bam',
            '/fake/path/to/IonXpress_007_rawlib.bam',
            '/fake/path/to/IonXpress_008_rawlib.bam',
            '/fake/path/to/IonXpress_009_rawlib.bam',
            '/fake/path/to/IonXpress_010_rawlib.bam',
            '/fake/path/to/IonXpress_011_rawlib.bam',
            '/fake/path/to/IonXpress_012_rawlib.bam',
            '/fake/path/to/IonXpress_013_rawlib.bam',
            '/fake/path/to/IonXpress_014_rawlib.bam',
            '/fake/path/to/IonXpress_015_rawlib.bam',
            '/fake/path/to/IonXpress_016_rawlib.bam',
            '/fake/path/to/nomatch_rawlib.bam'
        ]
        observed_result = bc_filter.filter_files(bam_list, barcode_filter)
        self.assertListEqual(expected_result, observed_result)

    def test_filter_files_from_file(self):
        expected_result = [
            '/fake/path/to/nomatch_rawlib.bam',
            '/fake/path/to/IonXpress_001_rawlib.bam',
            '/fake/path/to/IonXpress_003_rawlib.bam',
            '/fake/path/to/IonXpress_005_rawlib.bam',
            '/fake/path/to/IonXpress_007_rawlib.bam',
            '/fake/path/to/IonXpress_009_rawlib.bam',
            '/fake/path/to/IonXpress_011_rawlib.bam',
            '/fake/path/to/IonXpress_013_rawlib.bam',
            '/fake/path/to/IonXpress_015_rawlib.bam'
        ]
        barcode_filter = bc_filter.load_filter_from_file(
            os.path.join(os.path.dirname(__file__), 'test_data', 'test_barcodeFilter.txt')
        )
        bam_list = [
            '/fake/path/to/IonXpress_001_rawlib.bam',
            '/fake/path/to/IonXpress_002_rawlib.bam',
            '/fake/path/to/IonXpress_003_rawlib.bam',
            '/fake/path/to/IonXpress_004_rawlib.bam',
            '/fake/path/to/IonXpress_005_rawlib.bam',
            '/fake/path/to/IonXpress_006_rawlib.bam',
            '/fake/path/to/IonXpress_007_rawlib.bam',
            '/fake/path/to/IonXpress_008_rawlib.bam',
            '/fake/path/to/IonXpress_009_rawlib.bam',
            '/fake/path/to/IonXpress_010_rawlib.bam',
            '/fake/path/to/IonXpress_011_rawlib.bam',
            '/fake/path/to/IonXpress_012_rawlib.bam',
            '/fake/path/to/IonXpress_013_rawlib.bam',
            '/fake/path/to/IonXpress_014_rawlib.bam',
            '/fake/path/to/IonXpress_015_rawlib.bam',
            '/fake/path/to/IonXpress_016_rawlib.bam',
            '/fake/path/to/nomatch_rawlib.bam'
        ]
        observed_result = bc_filter.filter_files(bam_list, barcode_filter)
        self.assertListEqual(expected_result, observed_result)
